/********************************************************
 * File: scan.c
 * Created at Sun Jan 28 22:10:32 MSK 2001 by raorn // raorn@binec.ru
 * SCAN
 * $Id: scan.c,v 1.8 2002/01/10 02:04:09 raorn Exp $
 *******************************************************/
#include "crashecho.h"

void LogScanResults(void)
{
  printf("\n");

  if (scan_total == 0)
    LogWrite(2, TOSSINGINFO, "No messages exported");
  else if (scan_total == 1)
    LogWrite(2, TOSSINGINFO, "1 message exported");
  else {
    LogWrite(2, TOSSINGINFO, "%lu messages exported", scan_total);
  }
}

bool ScanHandle(struct MemMessage *mm)
{
  uchar buf[50];
  ulong oldattr;

  if (mm->Area[0] == 0) {
    Print4D(&mm->DestNode, buf, 50);

    LogWrite(4, TOSSINGINFO, "Exporting message #%lu from \"%s\" to \"%s\" at %s", mm->msgnum, mm->From, mm->To, buf);
  } else {
    LogWrite(4, TOSSINGINFO, "Exporting message #%lu from \"%s\" to \"%s\" in %s", mm->msgnum, mm->From, mm->To, mm->Area);
  }

  if (config.cfg_Flags & CFG_ADDTID)
    AddTID(mm);

  oldattr = mm->Attr;

  if (HandleMessage(mm, FALSE)) {
    scan_total++;

    mm->Attr = oldattr;
    mm->Changed = TRUE;

    if (mm->Area[0] == 0) {
      if ((config.cfg_Flags & CFG_ALLOWKILLSENT) && (mm->Attr & FLAG_KILLSENT)) {
        LogWrite(2, TOSSINGINFO, "Deleting message with KILLSENT flag");
        mm->Deleted = TRUE;
      } else {
        mm->Attr |= FLAG_SENT;
      }
    } else {
      mm->Attr |= FLAG_SENT;
    }

    return TRUE;
  }

  return FALSE;
}

bool ScanArea(uchar * tagname)
{
  struct Area *area;

  for (area = (struct Area *) config.AreaList.First; area; area = area->Next)
    if (stricmp(area->Tagname, tagname) == 0)
      break;

  if (area) {
    if (area->AreaType != AREATYPE_ECHOMAIL && area->AreaType != AREATYPE_NETMAIL) {
      LogWrite(1, USERERR, "You cannot scan area %s, not an echomail or netmail area", tagname);
    } else if (!area->Messagebase) {
      LogWrite(1, USERERR, "You cannot scan area %s, area is pass-through", tagname);
    } else if (!area->Messagebase->scanfunc) {
      LogWrite(1, USERERR, "You cannot scan area %s, scanning is not supported for this type of messagebase", tagname);
    } else {
      printf("Scanning area %s\n", area->Tagname);
      return (*area->Messagebase->scanfunc) (area, TRUE, FALSE, 0, AcceptNotSent, ScanHandle);
    }
  } else {
    LogWrite(1, USERERR, "Unknown area %s", tagname);
  }

  return (FALSE);
}

bool Scan(void)
{
  struct Area *area;

  LogWrite(2, ACTIONINFO, "Scanning areas for messages to export");

  for (area = (struct Area *) config.AreaList.First; area; area = area->Next)
    if (area->Messagebase && (area->AreaType == AREATYPE_ECHOMAIL || area->AreaType == AREATYPE_NETMAIL)) {
      if (area->Messagebase->scanfunc) {
        printf("Scanning area %s\n", area->Tagname);

        if (!(*area->Messagebase->scanfunc) (area, TRUE, FALSE, 0, AcceptNotSent, ScanHandle)) {
          return (FALSE);
        }
      }
    }

  return (TRUE);
}

bool ScanList(uchar * file)
{
  FILE *fh;
  uchar buf[100];

  if (!(fh = fopen(file, "rt"))) {
    LogWrite(1, USERERR, "Unable to open %s", file);
    return (FALSE);
  }

  while (fgets(buf, 100, fh)) {
    if (strlen(buf) > 2) {
      if (buf[strlen(buf) - 1] < 32)
        buf[strlen(buf) - 1] = 0;

      if (!ScanArea(buf)) {
        fclose(fh);
        return (FALSE);
      }
    }
  }

  fclose(fh);

  return (TRUE);
}
